@@ -71,7 +71,7 @@ def filter_and_return_output(dump, bufsize=0, *varargs):
   # Since we call svntest.main.run_command_stdin() in binary mode,
   # normalize the stderr line endings on Windows ourselves.
   if sys.platform == 'win32':
-    errput = map(lambda x : x.replace('\r\n', '\n'), errput)
+    errput = [x.replace('\r\n', '\n') for x in errput]
 
   return output, errput
 
