@@ -37,6 +37,7 @@
 #include "svn_error.h"
 #include "svn_config.h"
 #include "private/svn_subr_private.h"
+#include "private/svn_config_private.h"
 
 #include "../svn_test.h"
 
@@ -69,12 +70,12 @@ get_config_file_path(const char **cfg_file,
 }
 
 static const char *config_keys[] = { "foo", "a", "b", "c", "d", "e", "f", "g",
-                                     "h", "i", NULL };
+                                     "h", "i", "m", NULL };
 static const char *config_values[] = { "bar", "Aa", "100", "bar",
                                        "a %(bogus)s oyster bar",
                                        "%(bogus)s shmoo %(",
                                        "%Aa", "lyrical bard", "%(unterminated",
-                                       "Aa 100", NULL };
+                                       "Aa 100", "foo bar baz", NULL };
 
 static svn_error_t *
 test_text_retrieval(const svn_test_opts_t *opts,
@@ -367,6 +368,18 @@ test_expand(const svn_test_opts_t *opts,
      of "c" was not created in a temporary pool when expanding "g". */
   SVN_TEST_STRING_ASSERT(val, "bar");
 
+  /* Get expanded "j" and "k" which have cyclic definitions.
+   * They must return empty values. */
+  svn_config_get(cfg, &val, "section1", "j", NULL);
+  SVN_TEST_STRING_ASSERT(val, "");
+  svn_config_get(cfg, &val, "section1", "k", NULL);
+  SVN_TEST_STRING_ASSERT(val, "");
+
+  /* Get expanded "l" which depends on a cyclic definition.
+   * So, it also considered "undefined" and will be normalized to "". */
+  svn_config_get(cfg, &val, "section1", "l", NULL);
+  SVN_TEST_STRING_ASSERT(val, "");
+
   return SVN_NO_ERROR;
 }
 
@@ -391,6 +404,60 @@ test_invalid_bom(apr_pool_t *pool)
   return SVN_NO_ERROR;
 }
 
+static svn_error_t *
+test_serialization(apr_pool_t *pool)
+{
+  svn_stringbuf_t *original_content;
+  svn_stringbuf_t *written_content;
+  svn_config_t *cfg;
+
+  const struct
+    {
+      const char *section;
+      const char *option;
+      const char *value;
+    } test_data[] =
+    {
+      { "my section", "value1", "some" },
+      { "my section", "value2", "something" },
+      { "another Section", "value1", "one" },
+      { "another Section", "value2", "two" },
+      { "another Section", "value 3", "more" },
+    };
+  int i;
+
+  /* Format the original with the same formatting that the writer will use. */
+  original_content = svn_stringbuf_create("\n[my section]\n"
+                                          "value1=some\n"
+                                          "value2=%(value1)sthing\n"
+                                          "\n[another Section]\n"
+                                          "value1=one\n"
+                                          "value2=two\n"
+                                          "value 3=more\n",
+                                          pool);
+  written_content = svn_stringbuf_create_empty(pool);
+
+  SVN_ERR(svn_config_parse(&cfg,
+                           svn_stream_from_stringbuf(original_content, pool),
+                           TRUE, TRUE, pool));
+  SVN_ERR(svn_config__write(svn_stream_from_stringbuf(written_content, pool),
+                            cfg, pool));
+  SVN_ERR(svn_config_parse(&cfg,
+                           svn_stream_from_stringbuf(written_content, pool),
+                           TRUE, TRUE, pool));
+
+  /* The serialized and re-parsed config must have the expected contents. */
+  for (i = 0; i < sizeof(test_data) / sizeof(test_data[0]); ++i)
+    {
+      const char *val;
+      svn_config_get(cfg, &val, test_data[i].section, test_data[i].option,
+                     NULL);
+      SVN_TEST_STRING_ASSERT(val, test_data[i].value);
+    }
+
+  return SVN_NO_ERROR;
+}
+
 /*
    ====================================================================
    If you add a new test to this file, update this array.
@@ -425,6 +492,8 @@ static struct svn_test_descriptor_t test_funcs[] =
                        "test variable expansion"),
     SVN_TEST_PASS2(test_invalid_bom,
                    "test parsing config file with invalid BOM"),
+    SVN_TEST_PASS2(test_serialization,
+                   "test writing a config"),
     SVN_TEST_NULL
   };
 
